import unittest
from unittest.mock import patch

from lifeguard.http_client import post, get


class TestHttpClient(unittest.TestCase):
    @patch("lifeguard.http_client.HTTP_PROXY", "http_proxy")
    @patch("lifeguard.http_client.requests")
    def test_call_post_with_http_proxy(self, mock_requests):
        post("url", data="data", headers="headers")
        mock_requests.post.assert_called_with(
            url="url",
            headers="headers",
            data="data",
            auth=None,
            proxies={"http": "http_proxy"},
        )

    @patch("lifeguard.http_client.HTTPS_PROXY", "https_proxy")
    @patch("lifeguard.http_client.requests")
    def test_call_post_with_https_proxy(self, mock_requests):
        post("url", data="data", headers="headers")
        mock_requests.post.assert_called_with(
            url="url",
            headers="headers",
            data="data",
            auth=None,
            proxies={"https": "https_proxy"},
        )

    @patch("lifeguard.http_client.requests")
    def test_call_post_without_proxy(self, mock_requests):
        post("url", data="data", headers="headers")
        mock_requests.post.assert_called_with(
            url="url", headers="headers", data="data", auth=None
        )

    @patch("lifeguard.http_client.requests")
    def test_call_post_with_auth(self, mock_requests):
        post("url", data="data", headers="headers", auth=("user", "passwd"))
        mock_requests.post.assert_called_with(
            url="url", headers="headers", data="data", auth=("user", "passwd")
        )

    @patch("lifeguard.http_client.HTTP_PROXY", "http_proxy")
    @patch("lifeguard.http_client.requests")
    def test_call_get_with_http_proxy(self, mock_requests):
        get("url", headers="headers")
        mock_requests.get.assert_called_with(
            url="url", headers="headers", auth=None, proxies={"http": "http_proxy"}
        )

    @patch("lifeguard.http_client.HTTPS_PROXY", "https_proxy")
    @patch("lifeguard.http_client.requests")
    def test_call_get_with_https_proxy(self, mock_requests):
        get("url", headers="headers")
        mock_requests.get.assert_called_with(
            url="url", headers="headers", auth=None, proxies={"https": "https_proxy"}
        )

    @patch("lifeguard.http_client.requests")
    def test_call_get_without_proxy(self, mock_requests):
        get("url", headers="headers")
        mock_requests.get.assert_called_with(url="url", headers="headers", auth=None)

    @patch("lifeguard.http_client.requests")
    def test_call_get_with_auth(self, mock_requests):
        get("url", headers="headers", auth=("user", "password"))
        mock_requests.get.assert_called_with(
            url="url", headers="headers", auth=("user", "password")
        )



"""
Implementation of contollers helpers
"""
import os
import sys
import traceback
from functools import wraps
from os.path import join

import jinja2
from flask import Blueprint
from flask import Response as FlaskResponse
from flask import request as flask_request
from flask import session as flask_session
from flask import redirect as flask_redirect

from lifeguard.auth import AUTHENTICATION_METHODS
from lifeguard.context import LIFEGUARD_CONTEXT
from lifeguard.logger import lifeguard_logger as logger
from lifeguard.settings import LIFEGUARD_DIRECTORY
from lifeguard.utils import build_import

custom_controllers = Blueprint("custom", __name__)


def build_content_from_template(template, searchpath, data=None):
    if not data:
        data = {}

    template_loader = jinja2.FileSystemLoader(searchpath=searchpath)
    template_env = jinja2.Environment(loader=template_loader)
    template = template_env.get_template(template)
    return template.render(**data)


def render_template(template, data=None, headers=None, searchpath=None):
    """
    render template used in contollers
    """

    if not searchpath:
        searchpath = str(join(LIFEGUARD_DIRECTORY, "templates"))

    response = Response()
    response.template = template
    response.template_searchpath = searchpath
    response.data = data
    response.headers = headers
    return response


def send_status(status):
    response = Response()
    response.status = status
    return response


class Session:
    def __getitem__(self, key):
        return flask_session[key]

    def __setitem__(self, key, value):
        flask_session[key] = value

    def get(self, key):
        return flask_session.get(key)


class Request:
    @property
    def json(self):
        return flask_request.json

    @property
    def data(self):
        return flask_request.data

    @property
    def headers(self):
        return flask_request.headers

    @property
    def form(self):
        return flask_request.form

    @property
    def method(self):
        return flask_request.method

    @property
    def args(self):
        return flask_request.args

    @property
    def values(self):
        return flask_request.values

    @property
    def cookies(self):
        return flask_request.cookies


class Response:
    def __init__(self):
        self._content_type = "text/html"
        self._content = None
        self._template = None
        self._template_searchpath = None
        self._status = 200
        self._data = {}
        self._headers = {}
        self._cookies = {}

    @property
    def content_type(self):
        return self._content_type

    @content_type.setter
    def content_type(self, value):
        self._content_type = value

    @property
    def template(self):
        return self._template

    @template.setter
    def template(self, value):
        self._template = value

    @property
    def template_searchpath(self):
        return self._template_searchpath

    @template_searchpath.setter
    def template_searchpath(self, value):
        self._template_searchpath = value

    @property
    def status(self):
        return self._status

    @status.setter
    def status(self, value):
        self._status = value

    @property
    def content(self):
        return self._content

    @content.setter
    def content(self, value):
        self._content = value

    @property
    def data(self):
        return self._data

    @data.setter
    def data(self, value):
        self._data = value

    @property
    def headers(self):
        return self._headers

    @headers.setter
    def headers(self, value):
        self._headers = value

    @property
    def cookies(self):
        return self._cookies

    @cookies.setter
    def cookies(self, value):
        self._cookies = value

    def set_cookie(self, key, value, options=None):
        if not options:
            options = {}
        options["value"] = value
        self._cookies[key] = options


def redirect(location, code=302):
    return flask_redirect(location, code)


def login_required(function):
    """
    Decorator for login
    """

    @wraps(function)
    def wrapped(*args, **kwargs):
        if LIFEGUARD_CONTEXT.auth_method in AUTHENTICATION_METHODS:
            return AUTHENTICATION_METHODS[LIFEGUARD_CONTEXT.auth_method](
                args, kwargs, function
            )
        return function(*args, **kwargs)

    return wrapped


def register_custom_controller(path, function, options):
    endpoint = options.pop("endpoint", function.__name__)
    skip_login = options.pop("skip_login", False)

    if not skip_login:
        function = login_required(function)

    custom_controllers.add_url_rule(
        path, endpoint, configure_controller(function), **options
    )


def load_custom_controllers():
    """
    Load custom controllers from application path
    """
    sys.path.append(LIFEGUARD_DIRECTORY)

    if not os.path.exists(os.path.join(LIFEGUARD_DIRECTORY, "controllers")):
        return
    for root, _dirs, files in os.walk(os.path.join(LIFEGUARD_DIRECTORY, "controllers")):
        root = os.path.relpath(root, os.path.join(LIFEGUARD_DIRECTORY))
        for controller_file in files:
            if controller_file.endswith("_controller.py"):
                controller_module_name = (
                    f'{build_import(root, controller_file.replace(".py", ""))}'
                )
                logger.info(
                    "loading custom controller %s", controller_file.replace(".py", "")
                )

                module = "%s" % (controller_module_name)
                if module not in sys.modules:
                    __import__(module)


def treat_response(response):
    content = response.content
    if response.template:
        content = build_content_from_template(
            response.template, response.template_searchpath, data=response.data
        )

    flask_response = FlaskResponse(
        content,
        content_type=response.content_type,
        status=response.status,
        headers=response.headers,
    )

    for key in response.cookies.keys():
        value = response.cookies[key].pop("value")
        flask_response.set_cookie(key, value, **response.cookies[key])

    return flask_response


def configure_controller(function):
    @wraps(function)
    def wrapped(*args, **kwargs):
        try:
            response = function(*args, **kwargs)

            if isinstance(response, Response):
                return treat_response(response)
            return response
        except Exception as error:
            logger.error(
                "error on render dashboard index: %s",
                str(error),
                extra={"traceback": traceback.format_exc()},
            )

    return wrapped


def controller(path, **options):
    """
    Decorator to configure a custom controller
    """

    def function_reference(decorated):
        @wraps(decorated)
        def wrapped(*args, **kwargs):
            return decorated(*args, **kwargs)

        register_custom_controller(path, decorated, options)
        return wrapped

    return function_reference


request = Request()
session = Session()



"""
Action to send problem
"""
import smtplib
import traceback
from datetime import datetime

from lifeguard import NORMAL, PROBLEM
from lifeguard.context import LIFEGUARD_CONTEXT
from lifeguard.logger import lifeguard_logger as logger
from lifeguard.settings import (
    LIFEGUARD_EMAIL_SMTP_PASSWD,
    LIFEGUARD_EMAIL_SMTP_PORT,
    LIFEGUARD_EMAIL_SMTP_SERVER,
    LIFEGUARD_EMAIL_SMTP_USER,
    LIFEGUARD_PUBLIC_ADDRESS,
)

EMAIL_NOTIFICATIONS = {}


class Message:
    """
    Email render template
    """

    def __init__(self, receivers, subject, message, template):
        self.receivers = receivers
        self.subject = subject
        self.message = message
        self.template = template

    def render(self):
        """
        Render email to send
        """
        receivers_line = ",".join(
            [
                "{} <{}>".format(receiver["name"], receiver["email"])
                for receiver in self.receivers
            ]
        )
        return (
            self.template.replace("[[SMTP_USER]]", LIFEGUARD_EMAIL_SMTP_USER)
            .replace("[[RECEIVERS]]", receivers_line)
            .replace("[[SUBJECT]]", self.subject)
            .replace("[[MESSAGE]]", self.message)
            .replace("[[LIFEGUARD_PUBLIC_ADDRESS]]", LIFEGUARD_PUBLIC_ADDRESS)
        )


def send_email(validation_response, settings):
    """
    Send email action
    """

    if validation_response.status in settings["email"].get(
        "remove_from_sent_list_when", [NORMAL]
    ) and (validation_response.validation_name in EMAIL_NOTIFICATIONS):
        EMAIL_NOTIFICATIONS.pop(validation_response.validation_name)

    if (
        validation_response.status in settings["email"].get("send_in", [PROBLEM])
        and validation_response.validation_name not in EMAIL_NOTIFICATIONS
    ):
        EMAIL_NOTIFICATIONS[validation_response.validation_name] = {
            "sent_at": datetime.now()
        }

        message = Message(
            settings["email"]["receivers"],
            settings["email"]["subject"],
            str(validation_response.details),
            LIFEGUARD_CONTEXT.alert_email_template,
        )

        emails = [receiver["email"] for receiver in settings["email"]["receivers"]]
        logger.info("sending email %s to %s", message.render(), emails)
        try:
            session = smtplib.SMTP(
                LIFEGUARD_EMAIL_SMTP_SERVER, port=LIFEGUARD_EMAIL_SMTP_PORT
            )
            session.starttls()
            session.login(LIFEGUARD_EMAIL_SMTP_USER, LIFEGUARD_EMAIL_SMTP_PASSWD)
            session.sendmail(LIFEGUARD_EMAIL_SMTP_USER, emails, message.render())
            session.quit()
        except:
            logger.error(
                "error on send email %s to %s",
                message.render(),
                emails,
                extra={"traceback": traceback.format_exc()},
            )



import unittest
from unittest.mock import MagicMock, mock_open, patch, call

from lifeguard.controllers.assets import (
    send_file,
    send_css,
    send_js,
    send_img,
    load_assets_controllers,
)


class TestAssetsControllers(unittest.TestCase):
    @patch("lifeguard.controllers.assets.join")
    @patch("lifeguard.controllers.assets.io")
    def test_send_file(self, mock_io, mock_join):
        with patch("builtins.open", new_callable=mock_open()) as mock:
            mock_join.return_value = ""

            mock_io.BytesIO.return_value = "io"

            mock_file = MagicMock(instance="file")
            mock_file.read.return_value = "".encode("utf-8")
            mock.return_value.__enter__.return_value = mock_file

            response = send_file("file", "image/jpeg")

            self.assertDictEqual(
                response.__dict__,
                {
                    "_content": "io",
                    "_content_type": "image/jpeg",
                    "_cookies": {},
                    "_data": {},
                    "_headers": {},
                    "_status": 200,
                    "_template": None,
                    "_template_searchpath": None,
                },
            )

            mock.assert_called_with("", "rb")
            mock_join.assert_called_with(".", "public", "file")
            mock_io.BytesIO.assert_called_with(b"")

    @patch("lifeguard.controllers.assets.send_file")
    @patch("lifeguard.controllers.assets.logger")
    def test_send_css(self, mock_logger, mock_send_file):
        mock_send_file.return_value = ""

        response = send_css("file.css")

        self.assertEqual(response, "")
        mock_send_file.assert_called_with("css/file.css", "text/css")
        mock_logger.info.assert_called_with("returning css %s", "file.css")

    @patch("lifeguard.controllers.assets.send_file")
    def test_send_css_not_found(self, mock_send_file):
        mock_send_file.side_effect = Exception("exception")

        response = send_css("file.css")

        self.assertEqual(
            response.__dict__,
            {
                "_content": None,
                "_content_type": "text/html",
                "_cookies": {},
                "_data": {},
                "_headers": {},
                "_status": 404,
                "_template": None,
                "_template_searchpath": None,
            },
        )

    @patch("lifeguard.controllers.assets.send_file")
    @patch("lifeguard.controllers.assets.logger")
    def test_send_js(self, mock_logger, mock_send_file):
        mock_send_file.return_value = ""

        response = send_js("file.js")

        self.assertEqual(response, "")
        mock_send_file.assert_called_with("js/file.js", "application/javascript")
        mock_logger.info.assert_called_with("returning js %s", "file.js")

    @patch("lifeguard.controllers.assets.send_file")
    def test_send_js_not_found(self, mock_send_file):
        mock_send_file.side_effect = Exception("exception")

        response = send_js("file.js")

        self.assertEqual(
            response.__dict__,
            {
                "_content": None,
                "_content_type": "text/html",
                "_cookies": {},
                "_data": {},
                "_headers": {},
                "_status": 404,
                "_template": None,
                "_template_searchpath": None,
            },
        )

    @patch("lifeguard.controllers.assets.send_file")
    @patch("lifeguard.controllers.assets.logger")
    def test_send_img(self, mock_logger, mock_send_file):
        mock_send_file.return_value = ""

        response = send_img("file.jpg")

        self.assertEqual(response, "")
        mock_send_file.assert_called_with("img/file.jpg", "image/jpeg")
        mock_logger.info.assert_called_with("returning img %s", "file.jpg")

    @patch("lifeguard.controllers.assets.send_file")
    def test_send_img_not_found(self, mock_send_file):
        mock_send_file.side_effect = Exception("exception")

        response = send_img("file.jpg")

        self.assertEqual(
            response.__dict__,
            {
                "_content": None,
                "_content_type": "text/html",
                "_cookies": {},
                "_data": {},
                "_headers": {},
                "_status": 404,
                "_template": None,
                "_template_searchpath": None,
            },
        )

    @patch("lifeguard.controllers.assets.register_custom_controller")
    def test_load_assets_controllers(self, mock_register_custom_controller):
        load_assets_controllers()
        mock_register_custom_controller.assert_has_calls(
            [
                call("/img/<path:path>", send_img, {"methods": ["GET"]}),
                call("/js/<path:path>", send_js, {"methods": ["GET"]}),
                call("/css/<path:path>", send_css, {"methods": ["GET"]}),
            ]
        )



from setuptools import find_packages, setup
from pathlib import Path

this_directory = Path(__file__).parent
long_description = (this_directory / "README.md").read_text()

setup(
    name="lifeguard",
    version="1.0.3",
    url="https://github.com/LifeguardSystem/lifeguard",
    author="Diego Rubin",
    author_email="contact@diegorubin.dev",
    license="GPL2",
    scripts=["bin/lifeguard"],
    include_package_data=True,
    description="Application to monitor your systems and give you the security to sleep peacefully at night",
    long_description=long_description,
    long_description_content_type="text/markdown",
    install_requires=["flask", "schedule", "gunicorn", "requests", "tabulate"],
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "Intended Audience :: Information Technology",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Topic :: System :: Monitoring",
    ],
    packages=find_packages(),
)



import os


def generate_base_project():
    with open("lifeguard_settings.py", "w") as settings_file:
        settings_file.write(
            """
PLUGINS = []

def setup(_lifeguard_context):
    pass
"""
        )
    if not os.path.exists("validations"):
        os.makedirs("validations")



import os
import sys
import traceback
from json import JSONEncoder
from functools import wraps

from lifeguard.logger import lifeguard_logger as logger
from lifeguard.settings import (
    LIFEGUARD_DIRECTORY,
    LIFEGUARD_RUN_ONLY_VALIDATIONS,
    LIFEGUARD_SKIP_VALIDATIONS,
)
from lifeguard.statuses import PROBLEM, ACTION_STATUSES
from lifeguard.utils import build_import

VALIDATIONS = {}


def __execute_actions(actions, result, settings):
    for action in actions or []:
        logger.info(
            "executing action %s with result %s...",
            str(action.__name__),
            str(result),
        )
        action(result, settings)


class ValidationResponse:
    """
    Represents the result of a validation
    """

    def __init__(
        self, status, details, settings=None, last_execution=None, validation_name=None
    ):
        self._status = self.__validate_status(status)
        self._details = details
        self._settings = settings
        self._last_execution = last_execution
        self._validation_name = validation_name

    @property
    def validation_name(self):
        """
        Return validation name
        """
        return self._validation_name

    @property
    def status(self):
        """
        Return status
        """
        return self._status

    @property
    def details(self):
        """
        Return details
        """
        return self._details

    @property
    def settings(self):
        """
        Return settings
        """
        return self._settings

    @property
    def last_execution(self):
        """
        Return last execution
        """
        return self._last_execution

    @validation_name.setter
    def validation_name(self, value):
        """
        Setter for validation name
        """
        self._validation_name = value

    @last_execution.setter
    def last_execution(self, value):
        """
        Setter for last execution
        """
        self._last_execution = value

    def get_attributes(self):
        """
        Return all attributes in a dict.
        """
        attributes = {
            "validation_name": self.validation_name,
            "status": self.status,
            "details": self.details,
            "settings": self.settings,
        }

        if self.last_execution:
            attributes["last_execution"] = self.last_execution.strftime(
                "%Y-%m-%dT%H:%M"
            )
        return attributes

    def __str__(self):
        return str(self.get_attributes())

    def __validate_status(self, status):
        if status not in ACTION_STATUSES:
            raise ValueError(f"{status} is not a valid status")
        return status


class ValidationResponseEncoder(JSONEncoder):
    """
    Enconder for Validation Response
    """

    def default(self, validation_response):
        """
        Default implementation for validation response encoder
        """

        return validation_response.get_attributes()


def load_validations():
    """
    Load validations from application path
    """
    for root, _dirs, files in os.walk(os.path.join(LIFEGUARD_DIRECTORY, "validations")):
        root = os.path.relpath(root, os.path.join(LIFEGUARD_DIRECTORY))
        for validation_file in files:
            if validation_file.endswith("_validation.py"):
                validation_module_name = (
                    f'{build_import(root, validation_file.replace(".py", ""))}'
                )
                logger.info("loading validation %s", validation_file.replace(".py", ""))

                module = "%s" % (validation_module_name)
                if module not in sys.modules:
                    __import__(module)


def validation(
    description=None, actions=None, schedule=None, settings=None, actions_on_error=None
):
    """
    Decorator to configure a validation
    """
    if not settings:
        settings = {}

    def function_reference(decorated):
        @wraps(decorated)
        def wrapped(*args, **kwargs):
            try:
                if LIFEGUARD_RUN_ONLY_VALIDATIONS and (
                    decorated.__name__ not in LIFEGUARD_RUN_ONLY_VALIDATIONS
                ):
                    logger.info(
                        "validation %s not in LIFEGUARD_RUN_ONLY_VALIDATIONS",
                        decorated.__name__,
                    )
                    return None

                if LIFEGUARD_SKIP_VALIDATIONS and (
                    decorated.__name__ in LIFEGUARD_SKIP_VALIDATIONS
                ):
                    logger.info(
                        "validation %s in LIFEGUARD_SKIP_VALIDATIONS",
                        decorated.__name__,
                    )
                    return None

                result = decorated(*args, **kwargs)
                result.validation_name = decorated.__name__
                __execute_actions(actions, result, settings)

                return result
            except Exception as exception:
                logger.error(
                    "validation error %s: %s",
                    str(decorated.__name__),
                    str(exception),
                    extra={"traceback": traceback.format_exc()},
                )
                __execute_actions(
                    actions_on_error,
                    ValidationResponse(
                        PROBLEM,
                        {
                            "exception": str(exception),
                            "traceback": traceback.format_exc(),
                            "use_error_template": True,
                        },
                        validation_name=decorated.__name__,
                    ),
                    settings,
                )

        VALIDATIONS[decorated.__name__] = {
            "ref": wrapped,
            "description": description,
            "actions": actions,
            "schedule": schedule,
            "settings": settings,
        }

        return wrapped

    return function_reference
